// -----------------------------------------------------------------------------
//
// Author(s):     Peter Schraut (www.console-dev.de)
// Last Modified: 27th July 2005
//
// Purpose:       Demo shows how to use the fade and text functions
//
// -----------------------------------------------------------------------------
#include <hel.h>


// -----------------------------------------------------------------------------
// Global Variables
// -----------------------------------------------------------------------------
TCustomTextInfo g_TextInfo =
{
    // Backgroundnumber
    2,
    
    // That's the order of chars how they
    // are located on the source graphic!
    "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmn"\
    "opqrstuvwxyz0123456789?,.-;: #^+*@"\
    "!\"%\%&/()=?|\\<>[]{}"
};

TFadeInfo g_FadeInfo;
u8 g_NewFrame=TRUE;
u16 g_Angle=10;


// -----------------------------------------------------------------------------
// External Data
// -----------------------------------------------------------------------------
extern const unsigned char charset_Map[120];
extern const unsigned short charset_Palette[256];
extern const unsigned char charset_Tiles[7680];


// -----------------------------------------------------------------------------
// Prototypes
// -----------------------------------------------------------------------------
void VBLInterruptHandler(void);



// -----------------------------------------------------------------------------
// Program Entry Point
// -----------------------------------------------------------------------------
int main(void)
{
    ham_Init();
    hel_SysSetPrefetch(TRUE);
    hel_PadInit();
    ham_SetBgMode(1);
    
    // Initialize textdraw system
    hel_CustomTextInit(&g_TextInfo,                 // Pointer to custom_text_info
                       (void*)charset_Tiles,        // Pointer to the tile source data
                       SIZEOF_16BIT(charset_Tiles), // Size of the tiles to be copied into Tile RAM, in number of 16bit chunks
                       1,                           // Color mode of the tile set: 0 = 16colors/16 palettes 1 = 256 colors / 1 palette
                       0,                           // Palettenumber
                       MAP_SIZE_ROT_32X32,          // Size of the map (0-3). This depends on the map_rot parameter!
                       1);                          // 0 = normal map, 1 = rotation map


    // Turn on the background
    ham_InitBg(g_TextInfo.BgNo, BG_ACTIVE_ON, BG_PRIO_1, BG_MOSAIC_OFF);

    // Output a string
    hel_CustomTextPrint(&g_TextInfo,                // Pointer to custom_text_info structure
                        3,                          // X character number the string should start on
                        1,                          // Y character number the string should start on
                        "Palette and Text Example");// The string to output
    
    // Print moooore text :)
    hel_CustomTextPrint(&g_TextInfo, 8, 6, "O  O OOO O  ");
    hel_CustomTextPrint(&g_TextInfo, 8, 7, "O  O O   O  ");
    hel_CustomTextPrint(&g_TextInfo, 8, 8, "OOOO OOO O  ");
    hel_CustomTextPrint(&g_TextInfo, 8, 9, "O  O O   O  ");
    hel_CustomTextPrint(&g_TextInfo, 8,10, "O  O OOO OOO");
    hel_CustomTextPrint(&g_TextInfo, 2,14, "Press LEFT, RIGHT to rotate");
    hel_CustomTextPrint(&g_TextInfo, 6,16, "-A- to reset fader");
    
    // Init a palette fader
    g_FadeInfo = hel_FadeInit((void*)charset_Palette, // pPalette
                              FADE_PALETTE_BG,        // PalMode  (Palette mode, 0=bg palette, 1=obj palette)
                              FADE_COLORS_256,        // ColMode  (Color mode, 0=16 colors, 1=256 colors)
                              0);                     // PalNo (Palette Number)

    // Start VBL Interrupt
    hel_IntrStartHandler(INT_TYPE_VBL, (void*)&VBLInterruptHandler);
    
    
    for(;;)
    {
        // It's a new frame?
        if(g_NewFrame)
        {
            // Read current pad
            hel_PadReadState();

            static u8 FadeDone=FALSE;
            
            if(!FadeDone)
            {
               // Fade palette. returns true when it is done
               FadeDone = hel_FadePalette(&g_FadeInfo, 
                                         24); // amount per call (8 = 1 unit per go)
            }

            // Has the -A- button been pressed?
            if(hel_Pad.Pressed.A)
            {
               // YES! Now reset the fader
               hel_FadeReset(&g_FadeInfo);
               FadeDone=FALSE;
            }
            
            // Query joypad and either increase
            // or decrease the rotation-angle.
            if(hel_Pad.Held.Left) if(--g_Angle < 1) g_Angle=359;
            if(hel_Pad.Held.Right) if(++g_Angle > 359) g_Angle=0;
            
            // Rotate background
            ham_RotBg(g_TextInfo.BgNo, g_Angle, 120, 80, 256);
            
            
            g_NewFrame=FALSE;
        }
    }

    return 0;
}

// -----------------------------------------------------------------------------
// VBL Interupt Handler
// -----------------------------------------------------------------------------
void VBLInterruptHandler(void)
{
    g_NewFrame=TRUE;
}


