// -----------------------------------------------------------------------------
//
// Author(s):     Peter Schraut (www.console-dev.de)
// Last Modified: 16th July 2005
//
// Purpose:       Demo shows how to use hel_IntrStartHandler and
//                hel_IntrStopHandler functions.
//
// Notes:
//                Define USE_INTERRUPT_FUNCTIONS_FROM_HAM to
//                either use Interrupt-Functions from HEL or HAM in this demo.
//                This will show a bug in HAM 2.8 and earlier, which
//                stops all interrupts when you only stop the HBL.
//
// -----------------------------------------------------------------------------
#include <hel.h>

// -----------------------------------------------------------------------------
// Defines
// -----------------------------------------------------------------------------
//#define USE_INTERRUPT_FUNCTIONS_FROM_HAM

#ifdef USE_INTERRUPT_FUNCTIONS_FROM_HAM
  #define IntrStartHandler    ham_StartIntHandler
  #define IntrStopHandler     ham_StopIntHandler
#else
  #define IntrStartHandler    hel_IntrStartHandler
  #define IntrStopHandler     hel_IntrStopHandler
#endif

// -----------------------------------------------------------------------------
// Global Variables
// -----------------------------------------------------------------------------
u32 g_VblCounter=0;
u32 g_HblCounter=0;

// -----------------------------------------------------------------------------
// Prototypes
// -----------------------------------------------------------------------------
void VBLInterruptHandler(void);
void HBLInterruptHandler(void);


// -----------------------------------------------------------------------------
// Program Entry Point
// -----------------------------------------------------------------------------
int main(void)
{
    ham_Init();
    hel_SysSetPrefetch(TRUE);
    ham_SetBgMode(0);
    ham_InitText(0);

    IntrStartHandler(INT_TYPE_VBL, (void*)VBLInterruptHandler);
    IntrStartHandler(INT_TYPE_HBL, (void*)HBLInterruptHandler);
    
    for(;;)
    {
        hel_PadReadState();
        
        if(hel_Pad.Pressed.A)
          IntrStopHandler(INT_TYPE_HBL);
          
        if(hel_Pad.Pressed.B)
          IntrStartHandler(INT_TYPE_HBL, (void*)HBLInterruptHandler);
          
        // Show the counter so we see
        // if the interrupt is still triggered
        ham_DrawText(1, 1,"VBL: %d", g_VblCounter);
        ham_DrawText(1, 2,"HBL: %d", g_HblCounter);
    }

    return 0;
}

// -----------------------------------------------------------------------------
// VBL Interupt Handler
// -----------------------------------------------------------------------------
void VBLInterruptHandler(void)
{
    g_VblCounter++;
}

// -----------------------------------------------------------------------------
// HBL Interupt Handler
// -----------------------------------------------------------------------------
void HBLInterruptHandler(void)
{
    g_HblCounter++;
}


