// -----------------------------------------------------------------------------
//
// Author(s):     Peter Schraut (www.console-dev.de)
// Last Modified: 10th July 2005
//
// Purpose:       Demo shows how to use the map-system-camera.
//
// -----------------------------------------------------------------------------
#include <hel.h>


// -----------------------------------------------------------------------------
// Global Variables
// -----------------------------------------------------------------------------
u8  g_NewFrame=TRUE;
TMapScrollInfo g_MapInfo;


// X/Y Tile-Position (Key-Points)
const TMapCamKeyPoint CamKeyPoints[10]=
{
   {10, 5},
   {10,25},
   {15, 5},
   {15,25},
   { 0, 0},
   {40,10},
   {40,40},
   {20,40},
   {20,20},
   {0,  0}
};

// Map Cinema Scrolling Information
TMapCamScrollInfo g_MapCamInfo =
{
   (TMapCamKeyPoint*)&CamKeyPoints, // Pointer to keypoints
   10,                              // Amount of keypoints
   0                                // Start at keypoint 0
};


// -----------------------------------------------------------------------------
// External Data
// -----------------------------------------------------------------------------
extern const unsigned short racetrack_Map[16384];
extern const unsigned short racetrack_Palette[256];
extern const unsigned char racetrack_Tiles[9088];


// -----------------------------------------------------------------------------
// Prototypes
// -----------------------------------------------------------------------------
void VBLInterruptHandler(void);


// -----------------------------------------------------------------------------
// Program Entry Point
// -----------------------------------------------------------------------------
int main(void)
{
    ham_Init();
    hel_SysSetPrefetch(TRUE);
    ham_SetBgMode(0);
    
    // Load Tileset into VideoRAM
    ham_bg[0].ti = ham_InitTileSet((void*)&racetrack_Tiles, SIZEOF_16BIT(racetrack_Tiles), 1, 1);

    // Init the Map-System for Background 0
    g_MapInfo = hel_MapInit(0,        // Backgroundnumber
                            1024/8,   // MapWidth in tiles
                            1024/8,   // MapHeight in tiles
                            FALSE,    // RotationMap?
                            (void*)racetrack_Map);   // Source MapData

    // Load Background Palette
    hel_PalBgLoad256((void*)&racetrack_Palette);

    // Turn on Background
    ham_InitBg(0, TRUE, 0, FALSE);

    // Start VBL Interrupt
    ham_StartIntHandler(INT_TYPE_VBL, (void*)&VBLInterruptHandler);
    
    
    for(;;)
    {
        if(g_NewFrame)
        {
            static u8 CamScrollActive=TRUE;
            
            // Is the Camera-System still active?
            if(CamScrollActive)
            {
               // YES it is! Scroll the map to the next Keypoint.
               CamScrollActive = hel_MapScrollTo(&g_MapInfo, &g_MapCamInfo, 2, 2);
            }
            
            g_NewFrame=FALSE;
        }
    }
    
    return 0;
}


// -----------------------------------------------------------------------------
// VBL Interrupt Handler
// -----------------------------------------------------------------------------
void VBLInterruptHandler()
{
	g_NewFrame=TRUE;
}

