// -----------------------------------------------------------------------------
//
// Author(s):     Peter Schraut (www.console-dev.de)
// Last Modified: 10th July 2005
//
// Purpose:       Demo shows how to use the Custom Text System to output
//                strings letter by letter, like a typingmachine.
//
// -----------------------------------------------------------------------------
#include <hel.h>


// -----------------------------------------------------------------------------
// Global Variables
// -----------------------------------------------------------------------------
TCustomTextInfo g_TextInfo =
{
    // Backgroundnumber
    0,

    // That's the order of chars how they
    // are located on the source graphic!
    "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmn"\
    "opqrstuvwxyz0123456789?,.-;: #^+*@"\
    "!\"%\%&/()=?|\\<>[]{}"
};

u8 g_NewFrame=TRUE;


// -----------------------------------------------------------------------------
// External Data
// -----------------------------------------------------------------------------
extern const unsigned char charset_Map[120];
extern const unsigned short charset_Palette[256];
extern const unsigned char charset_Tiles[7680];


// -----------------------------------------------------------------------------
// Prototypes
// -----------------------------------------------------------------------------
void VBLInterruptHandler(void);



// -----------------------------------------------------------------------------
// Program Entry Point
// -----------------------------------------------------------------------------
int main(void)
{
    ham_Init();
    hel_SysSetPrefetch(TRUE);
    ham_SetBgMode(0);

    // Init Textdraw System
    hel_CustomTextInit(&g_TextInfo,                 // Pointer to TCustomTextInfo
                       (void*)charset_Tiles,        // Pointer to the tile source data
                       SIZEOF_16BIT(charset_Tiles), // Size of the tiles to be copied into Tile RAM, in number of 16bit chunks
                       1,                           // Color mode of the tile set: 0 = 16colors/16 palettes 1 = 256 colors / 1 palette
                       0,                           // Palettenumber
                       MAP_SIZE_32X32,              // Size of the map (0-3). This depends on the map_rot parameter!
                       0);                          // 0 = normal map, 1 = rotation map

    // Load Background Palette
    hel_PalBgLoad256((void*)charset_Palette);

    // Turn on the background
    ham_InitBg(g_TextInfo.BgNo, BG_ACTIVE_ON, BG_PRIO_1, BG_MOSAIC_OFF);

    // Start VBL Interrupt
    ham_StartIntHandler(INT_TYPE_VBL, (void*)&VBLInterruptHandler);
    
    
    for(;;)
    {
        if(g_NewFrame)
        {
            // hel_CustomTextType returns TRUE when it is done
            if(hel_CustomTextType(&g_TextInfo,                 // Pointer to custom_text_info structure
                                  4,                           // X character number the string should start on
                                  10,                          // Y character number the string should start on
                                  6,                           // Delay until the next character is displayed
                                  64,                          // Time to wait after text is entirely drawn
                                  "I am a typingmachine..."))  // The text itself
            {
               // It's done! Now clear the row again!
               hel_CustomTextClearRow(&g_TextInfo, 10);
            }
            
            g_NewFrame=FALSE;
        }
    }

    return 0;
}

// -----------------------------------------------------------------------------
// VBL Interrupt Handler
// -----------------------------------------------------------------------------
void VBLInterruptHandler(void)
{
    ham_CopyObjToOAM();
    g_NewFrame=TRUE;
}
/* END OF FILE */
