// -----------------------------------------------------------------------------
//
// Author(s):     Peter Schraut (www.console-dev.de)
// Last Modified: 10th July 2005
//
// Purpose:       Demo shows how to use the Map-System with Parallax Scrolling.
//
// Notes:         Levelgraphics borrowed from Nessie (http://nessie.gbadev.org)
//
// -----------------------------------------------------------------------------
#include <hel.h>


// -----------------------------------------------------------------------------
// Structures
// -----------------------------------------------------------------------------
typedef struct THero
{
  u8  ObjNo;
  s16 X, Y;
}THero;


// -----------------------------------------------------------------------------
// Defines
// -----------------------------------------------------------------------------
#define HERO_WIDTH      8
#define HERO_HEIGHT     8
#define COLLISION_NONE  1
#define COLLISION_BLOCK 0

// -----------------------------------------------------------------------------
// Global Variables
// -----------------------------------------------------------------------------
u8 g_NewFrame=TRUE;
THero g_Hero ATTR_MEM_IN_EWRAM;
TMapScrollInfo g_MapInfos[3] ATTR_MEM_IN_EWRAM;


// -----------------------------------------------------------------------------
// External Data
// -----------------------------------------------------------------------------
extern const unsigned short world2_layer0_Map[4096];
extern const unsigned short world2_layer0_Palette[16];
extern const unsigned char world2_layer0_Tiles[896];
extern const unsigned short world2_collision_Map[2054];
extern const unsigned short smallcircle_Palette[256];
extern const unsigned char smallcircle_Bitmap[64];

// -----------------------------------------------------------------------------
// Prototypes
// -----------------------------------------------------------------------------
void VBLInterruptHandler(void);



// -----------------------------------------------------------------------------
// Program Entry Point
// -----------------------------------------------------------------------------
int main(void)
{
    ham_Init();
    hel_SysSetPrefetch(TRUE);
    hel_PadInit();
    ham_SetBgMode(0);
    
    // Init 16 Color Tileset
    ham_bg[0].ti = ham_InitTileSet((void*)&world2_layer0_Tiles, SIZEOF_16BIT(world2_layer0_Tiles), 0, 1);

    // Init Map and CollisionMap
    g_MapInfos[0] = hel_MapInit(0, 30, 20, 0, (void*)world2_layer0_Map);
    g_MapInfos[1] = hel_MapInitVirtual(30, 20, sizeof(u16), (void*)world2_collision_Map);
    
    // Create the Hero Sprite. That's actually
    // only a small Circle!
    g_Hero.X = 16;
    g_Hero.Y = 16;
    g_Hero.ObjNo = hel_ObjCreate256((void*)smallcircle_Bitmap, OBJ_SIZE_8X8, 1, g_Hero.X, g_Hero.Y);
    
    // Load Background Palette
    hel_PalBgLoad16((void*)world2_layer0_Palette, 0);
    
    // Load Object Palette
    hel_PalObjLoad256((void*)smallcircle_Palette);
    
    // Turn on Background
    ham_InitBg(0, TRUE, 0, FALSE);
    
    // Start VBL Interrupt
    ham_StartIntHandler(INT_TYPE_VBL, (void*)&VBLInterruptHandler);
    
    
    for(;;)
    {
        if(g_NewFrame)
        {
            TPoint32 MoveBy={0,0};
            int AdjustX=0;
            int AdjustY=0;
            
            // Read current pad state
            hel_PadReadState();

            if(hel_Pad.Held.Down)
            {
                MoveBy.Y += 1;
                AdjustY = HERO_HEIGHT;
            }
            else
            if(hel_Pad.Held.Up)
            {
                MoveBy.Y -= 1;

            }

            if(hel_Pad.Held.Left)
            {
                MoveBy.X -= 1;
            }
            else
            if(hel_Pad.Held.Right)
            {
                MoveBy.X += 1;
                AdjustX = HERO_WIDTH;
            }


            // Get TileNumber of position where Hero wants to go to
            u16 TileNo = *(u16*)hel_MapGetTilePtrAt(&g_MapInfos[1], g_Hero.X + AdjustX + MoveBy.X, g_Hero.Y + AdjustY + MoveBy.Y);
            
            // Only set new position when
            // there is no collision!
            if(COLLISION_NONE == TileNo)
            {
              g_Hero.X += MoveBy.X;
              g_Hero.Y += MoveBy.Y;

              hel_ObjSetXY(g_Hero.ObjNo, g_Hero.X, g_Hero.Y);
            }

            g_NewFrame=FALSE;
        }
    }

    return 0;
}

// -----------------------------------------------------------------------------
// VBL Interrupt Handler
// -----------------------------------------------------------------------------
void VBLInterruptHandler()
{
  ham_CopyObjToOAM();
	g_NewFrame=TRUE;
}

