////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                                //
// HEL Library (c) 2003-2005 by Peter Schraut                                                     //
// http://www.console-dev.de                                                                      //
//                                                                                                //
// ---------------------------------------------------------------------------------------------- //
//                                                                                                //
// HEL Library is distributed as freeware.                                                        //
//                                                                                                //  
// You are free to use HEL Library as part of your program for any purpose including freeware,    //
// commercial and shareware programs, provided some credit is given to the author.                //
//                                                                                                // 
// The origin of this software must not be misrepresented; you must not claim your authorship.    //
// All redistributions must retain the original copyright notice and web site addresses.          //
//                                                                                                // 
// Commercial redistribution of the library is allowed only with an explicit written              // 
// permission from the author.                                                                    // 
//                                                                                                // 
// If you finished a commercial game using HEL library, the author would like to be informed      // 
// in what game it is used.                                                                       // 
//                                                                                                // 
// This software is provided 'as-is', without warranty of any kind, either expressed or implied.  // 
// In no event shall the author be held liable for any damages arising from the use of this       //
// software.                                                                                      // 
//                                                                                                // 
////////////////////////////////////////////////////////////////////////////////////////////////////
#ifndef HEL_H
#define HEL_H

#include <mygba.h>
#include <stdio.h>
#include <stdarg.h>
#include <malloc.h>

// Define HAM_DEBUGGER when generating the
// documentation using DoxyGen. This is only
// that DoxyGen uses the function documentation,
// since we do not have documented the macros
#ifdef DOXYGEN_SHOULD_SKIP_THIS
  #define HAM_DEBUGGER 1
#endif

//
// ---[ HEL version defines ]-----------------------------------------------------------------------
//

#define HEL_CURRENT_VERSION_MAJOR   1
#define HEL_CURRENT_VERSION_MINOR   6
#define HEL_CURRENT_VERSION_STRING  "1.6"
#define HEL_LIBRARY_STRING          "HEL Library "HEL_CURRENT_VERSION_STRING

//
// ---[ HEL Debug Message System ]------------------------------------------------------------------
//

//#define HEL_DEBUG_MSG_TYPE_VBA
#define HEL_DEBUG_MSG_TYPE_NOCHASH

#if defined(HEL_DEBUG_MSG_TYPE_VBA) && defined(HEL_DEBUG_MSG_TYPE_NOCHASH)
  #error
  #error HEL Library definition error occured!
  #error
  #error You must not define more than one Debug-Message-Type (HEL_DEBUG_MSG_TYPE_*)!
  #error
  #error Open ham/include/hel.h and either enable HEL_DEBUG_MSG_TYPE_VBA
  #error or(!) HEL_DEBUG_MSG_TYPE_NOCHASH, but not both!
  #error
  #error The Debug-Output-Messages work only when HAM_DEBUGGER is enabled. The HAM_DEBUGGER 
  #error define is located pretty much at the top in ham/include/mygba.h
  #error
#endif

#ifdef HAM_DEBUGGER
  #ifdef HEL_DEBUG_MSG_TYPE_VBA
    #define HEL_DEBUG_MSG(txt, args...)   ham_VBAText(txt,##args)
    #define HEL_DEBUG_BRK          
  #elif defined(HEL_DEBUG_MSG_TYPE_NOCHASH)
    #define HEL_DEBUG_MSG(txt, args...)   hel_DebugNoCashMessage(txt,##args)
    #define HEL_DEBUG_BRK                 __asm ("mov r11, r11 @ no$gba breakpoint\n")
  #else
    #define HEL_DEBUG_MSG(txt, args...)
    #define HEL_DEBUG_BRK
  #endif
#else
  #define HEL_DEBUG_MSG(txt, args...)
  #define HEL_DEBUG_BRK
#endif


//
// ---[ Puts "Something" only in Debugmode ]--------------------------------------------------------
//

#ifdef HAM_DEBUGGER
  #define HEL_DEBUG_PUT(Something) Something
#else
  #define HEL_DEBUG_PUT(Something) 
#endif


//
// ---[ Version checking macros for HAM ]-----------------------------------------------------------
//

//
// EQ=Equal, NE=NotEqual, GT=GreaterThan, GTE=GreaterThanEqual, LE=LessEqual, LT=LessThan
//
#define CURRENT_HAM_VERSION_EQ(major, minor)   (HAM_CURRENT_VERSION_MAJOR == (major) && HAM_CURRENT_VERSION_MINOR >= (minor))
#define CURRENT_HAM_VERSION_NE(major, minor)   (HAM_CURRENT_VERSION_MAJOR != (major) || HAM_CURRENT_VERSION_MINOR != (minor))
#define CURRENT_HAM_VERSION_GT(major, minor)   ((HAM_CURRENT_VERSION_MAJOR > (major))?TRUE:(HAM_CURRENT_VERSION_MAJOR == (major) && HAM_CURRENT_VERSION_MINOR > (minor))?TRUE:FALSE)
#define CURRENT_HAM_VERSION_GTE(major, minor)  ((HAM_CURRENT_VERSION_MAJOR > (major))?TRUE:(HAM_CURRENT_VERSION_MAJOR == (major) && HAM_CURRENT_VERSION_MINOR >= (minor))?TRUE:FALSE)
#define CURRENT_HAM_VERSION_LE(major, minor)   ((HAM_CURRENT_VERSION_MAJOR < (major))?TRUE:(HAM_CURRENT_VERSION_MAJOR == (major) && HAM_CURRENT_VERSION_MINOR <= (minor))?TRUE:FALSE)
#define CURRENT_HAM_VERSION_LT(major, minor)   ((HAM_CURRENT_VERSION_MAJOR < (major))?TRUE:(HAM_CURRENT_VERSION_MAJOR == (major) && HAM_CURRENT_VERSION_MINOR < (minor))?TRUE:FALSE)


//
// ---[ General Typedef's ]-------------------------------------------------------------------------
//

typedef struct TPoint8
{
   s8 X, Y;
}TPoint8, *PPoint8;

typedef struct TPointU8
{
   u8 X, Y;
}TPointU8, *PPointU8;

typedef struct TPoint16
{
   s16 X, Y;
}TPoint16, *PPoint16;

typedef struct TPointU16
{
   u16 X, Y;
}TPointU16, *PPointU16;

typedef struct TPoint32
{
   s32 X, Y;
}TPoint32, *PPoint32;

typedef struct TPointU32
{
   u32 X, Y;
}TPointU32, *PPointU32;

typedef struct TPointSfp16
{
  sfp16 X;
  sfp16 Y;
}TPointSfp16, *PPointSfp16;

typedef struct TPointSfp32
{
  sfp32 X;
  sfp32 Y;
}TPointSfp32, *PPointSfp32;

typedef struct
{
   s8 Left;
   s8 Right;
   s8 Top;
   s8 Bottom;
}TRect8;

typedef struct
{
   s16 Left;
   s16 Right;
   s16 Top;
   s16 Bottom;
}TRect16;

typedef struct
{
   s32 Left;
   s32 Right;
   s32 Top;
   s32 Bottom;
}TRect32;



//
// ---[ Defines ]-----------------------------------------------------------------------------------
//
#define EVEN(v)                        (0==((v)&1))
#define ODD(v)                         (!EVEN(v))
#define NOT                            !
#define SAME(a,b)                      ((a) == (b))
#define BIT(n)                         (1<<(n))
#define BIT_NOT(n)                     (~BIT(n))
#define BIT_IS_SET(v,b)                ((((v) >> (b)) & 1))
#define SHR(v,n)                       ((v) >> (n))
#define SHL(v,n)                       ((v) << (n))
#define MIN(a,b)                       (((a)<(b))?(a):(b))
#define MID(a,b,c)                     (MAX(MIN((a),(b)),MIN((a),(c)))
#define MAX(a,b)                       (((a)>(b))?(a):(b))
#define LIMIT(Value,ValueMin,ValueMax) (MIN(MAX((Value), (ValueMin)), (ValueMax)))
#define SGN(v)                         (((v) < 0)?-1:(((v) > 0)?(+1):(0)))
#define ABS(v)                         (((v) > 0)?(v):(-(v)))
#define ASSIGNED(p)                    (NULL != (p))
#define ACCESS_U8(v)                   *(volatile u8*)(v)
#define ACCESS_U16(v)                  *(volatile u16*)(v)
#define ACCESS_U32(v)                  *(volatile u32*)(v)
#define RANDOM(n)                      (rand()%(n))
#define SIN(n)                         (SIN_SFP32[(u32)(n)%360])
#define COS(n)                         (COS_SFP32[(u32)(n)%360])
#define ATTR_ALIGNED(n)                __attribute__ ((aligned(n)))
#define ATTR_PACKED                    __attribute__ ((packed))
#define ATTR_SECTION(name)             __attribute__ ((section(name)))
#define ATTR_MEM_IN_EWRAM              ATTR_SECTION(".ewram")
#define ATTR_MEM_IN_IWRAM              ATTR_SECTION(".iwram")
#define ATTR_FUNC_IN_IWRAM             __attribute__ ((section (".iwram"), long_call))
#define ATTR_DEPRECATED                __attribute__ ((deprecated))
#define ATTR_NOINLINE                  __attribute__ ((noinline))
#define ATTR_FORCEINLINE               __attribute__ ((always_inline))
#define ATTR_USED                      __attribute__ ((used))
#define FREE(p)                        { if(ASSIGNED(p)){ free(p); (p)=NULL; } }
#define EXTERN_DATA8(name,size)        extern const u8 name[size];
#define EXTERN_DATA16(name,size)       extern const u16 name[size];
#define EXTERN_DATA32(name,size)       extern const u32 name[size];
#define EXTERN_U8(name,size)           extern const unsigned char name[size];
#define EXTERN_U16(name,size)          extern const unsigned short name[size];
#define EXTERN_U32(name,size)          extern const unsigned long name[size];
#define INCBIN(file)                   __asm__ (".incbin \"" file "\"\n");
#define INCBIN_U8(array, file)         const unsigned char array[]={};  INCBIN(file);
#define INCBIN_U16(array, file)        const unsigned short array[]={}; INCBIN(file);
#define INCBIN_U32(array, file)        const unsigned long array[]={};  INCBIN(file);
#define ADDR(p)                        ((u32)(p))
#define ADDR_WORDALIGNED(p)            (((ADDR(p)&3))==0)
#define ADDR_HALFWORDALIGNED(p)        (((ADDR(p)&1))==0)

//
// Use it like:
//  TMyStruct *p;
//      p = NEW(TMyStruct, 10); // allocates memory for 10 TMyStruct's
//
// use HAM's malloc wrapper if HAM version >= 2.80 and HAM_DEBUGGER is enabled
#if CURRENT_HAM_VERSION_GTE(2,80) && HAM_DEBUGGER
   #define NEW(struct_, count)         (struct_*)ham_Malloc((count)*sizeof(struct_))
#else
   #define NEW(struct_, count)         (struct_*)malloc((count)*sizeof(struct_))
#endif


// VBAText stuff does -NOT- work on hardware!!!
// Hint: HEL_DEBUG_MSG instead!
#ifdef HAM_DEBUGGER
   #define VBAOUT(txt, args...)        ham_VBAText(txt,##args);
#else
   #define VBAOUT(txt, args...)
#endif


#define SET_POINT(P, X_, Y_)        \
   {                                \
      (P)->X = X_; (P)->Y = Y_;     \
   }

#define SET_RECT(P, Left_, Top_, Right_, Bottom_)  \
   {                                               \
      (P)->Left=Left_;   (P)->Top=Top_;            \
      (P)->Right=Right_;  (P)->Bottom=Bottom_;     \
   }




#ifndef __cplusplus
   #define false     0
   #define true      (!false)
   
   #ifndef bool
      #define bool    u8
   #endif 
   
   #ifndef null
      #define null    NULL
   #endif 
#endif


// These can be used with TOOL_DMA0/1/2/3_SET_EX
#define DMA_INCREMENT       (0)
#define DMA_DECREMENT       (1)
#define DMA_NOCHANGE        (2)
#define DMA_REPEAT_ON       (1)
#define DMA_REPEAT_OFF      (0)
#define DMA_IRQ_ON          (1)
#define DMA_IRQ_OFF         (0)


#if CURRENT_HAM_VERSION_LT(2,80)
   // can be used with ham_InitMapEmptySet(..)
   // normal map sizes
   #define MAP_SIZE_32X32        0
   #define MAP_SIZE_64X32        1
   #define MAP_SIZE_32X64        2
   #define MAP_SIZE_64X64        3
   
   // rotationmap sizes
   #define MAP_SIZE_ROT_16X16    0
   #define MAP_SIZE_ROT_32X32    1
   #define MAP_SIZE_ROT_64X64    2
   #define MAP_SIZE_ROT_128X128  3
#endif

// Helpers for hel_MapSetScrollFlags
// Use this to whether specify if the mapsystem is allowed
// to scroll or not. Each direction can be turned on and off.
#define MAP_SCROLLFLAGS_LEFT                           BIT0
#define MAP_SCROLLFLAGS_RIGHT                          BIT1
#define MAP_SCROLLFLAGS_UP                             BIT2
#define MAP_SCROLLFLAGS_DOWN                           BIT3
#define MAP_SCROLLFLAGS_TRANSMITPOSITION               BIT4
#define MAP_SCROLLFLAGS_DEFAULT                        (MAP_SCROLLFLAGS_LEFT | MAP_SCROLLFLAGS_RIGHT | MAP_SCROLLFLAGS_UP | MAP_SCROLLFLAGS_DOWN | MAP_SCROLLFLAGS_TRANSMITPOSITION)

// Passed to the MapDrawer
#define MAP_DRAWINFOFLAGS_SHOW                         BIT0
#define MAP_DRAWINFOFLAGS_HIDE                         BIT1

// Helpers for hel_MapJumpTo
#define MAP_JUMPTO_LEFT                                BIT0
#define MAP_JUMPTO_RIGHT                               BIT1
#define MAP_JUMPTO_TOP                                 BIT2
#define MAP_JUMPTO_BOTTOM                              BIT3

//#if CURRENT_HAM_VERSION_LT(2,80)
   #define OBJ_COLORS_16         0
   #define OBJ_COLORS_256        1
   //#define OBJ_INVALID_ID        0xff
//#endif

// Can be used for OBJ Creation Functions
#define OBJ_SHAPE_SQUARE         0
#define OBJ_SHAPE_HORIZONTAL     1
#define OBJ_SHAPE_VERTICAL       2


// Helper defines. Can be used for hel_Splash(..)
#define SPLASH_TYPE_BLACK        0
#define SPLASH_TYPE_WHITE        1


// Helper defines for hel_FadePal(..)
#define FADE_PALETTE_BG          0
#define FADE_PALETTE_OBJ         1
#define FADE_COLORS_16           0
#define FADE_COLORS_256          1


// Helper defines for windowing
#define WIN_FX                   (BIT5)
#define WIN_ALL                  (WIN_BG0 | WIN_BG1 | WIN_BG2 | WIN_BG3 | WIN_OBJ)
#define WIN_NONE                 (0)
// specify 0 or 1 to whether exclude or include window
#define WIN_SELECT(bg0, bg1, bg2, bg3, obj) ((bg0) | ((bg1) << WIN_BG1) | ((bg2) << WIN_BG2) | ((bg3) << WIN_BG3) | ((obj) << WIN_OBJ))


//
// ---[ SpecialFX Defines ]---------------------------------------------------------------------
//
// These defines can be used with ham_SetFxMode
//
#define FX_LAYER_BG0     BIT0
#define FX_LAYER_BG1     BIT1
#define FX_LAYER_BG2     BIT2
#define FX_LAYER_BG3     BIT3
#define FX_LAYER_OBJ     BIT4
#define FX_LAYER_BD      BIT5
#define FX_LAYER_ALL     0x3f
#define FX_LAYER_NONE    0x00


//
// ---[ Generic Screen Defines ]--------------------------------------------------------------------
//
#define SCREEN_DISPLAY_WIDTH              240
#define SCREEN_DISPLAY_HEIGHT             160

//
// ---[ Parallax macros ]---------------------------------------------------------------------------
//
#define PARALLAX_RATIO_QUARTER            0x3fff
#define PARALLAX_RATIO_HALF               0x7fff
#define PARALLAX_RATIO_THREEQUARTER       0xbfff
#define PARALLAX_RATIO_ONE                0xffff
#define PARALLAX_FLOAT_TO_RATIO(value)    CONV_FLOAT_TO_SFP32(value)
#define PARALLAX_RATIO_TO_FLOAT(value)    (float)((value)/(float)PARALLAX_RATIO_ONE)
#define PARALLAX_RATIO_TO_INT(value)      CONV_SFP32_TO_S16(value)


//
// ---[ Background macros ]-------------------------------------------------------------------------
//
#define R_BGXSCRLX(BgNo)                      ACCESS_16(MEM_IO + 0x10 + ((BgNo)<<2))
#define R_BGXSCRLY(BgNo)                      ACCESS_16(MEM_IO + 0x12 + ((BgNo)<<2))
#define M_BGXSCRLX_SET(BgNo, Value)           R_BGXSCRLX(BgNo) = Value;
#define M_BGXSCRLY_SET(BgNo, Value)           R_BGXSCRLY(BgNo) = Value;
#define M_BGXSCRLXY_SET(BgNo, ValueX, ValueY) { M_BGXSCRLX_SET(BgNo, ValueX); M_BGXSCRLY_SET(BgNo, ValueY); }

//
// ---[ Object Memory Location macros ]-------------------------------------------------------------
//
#define MEM_OBJ_SLOT(Slot)                    (MEM_OBJ+((Slot)<<5))
#define MEM_OBJ_SLOT_PTR(Slot)                ((u16*)MEM_OBJ_SLOT(Slot))


//
// ---[ Timer macros ]------------------------------------------------------------------------------
//
#define F_TIM0CNT_IRQ_ENABLED                ((R_TIM0CNT & BIT6) == BIT6)
#define F_TIM0CNT_TIMER_STARTED              ((R_TIM0CNT & BIT7) == BIT7)
#define M_TIM1CNT_COUNTUP_TIMING_ENABLE      { R_TIM1CNT &= NOT_BIT2; R_TIM1CNT |= BIT2; }
#define M_TIM1CNT_COUNTUP_TIMING_DISABLE     R_TIM1CNT &= NOT_BIT2;
#define F_TIM1CNT_COUNTUP_TIMING_ENABLED     ((R_TIM1CNT &= NOT_BIT2) == NOT_BIT2)
#define F_TIM1CNT_IRQ_ENABLED                ((R_TIM1CNT & BIT6) == BIT6)
#define F_TIM1CNT_TIMER_STARTED              ((R_TIM1CNT & BIT7) == BIT7)
#define M_TIM2CNT_COUNTUP_TIMING_ENABLE      { R_TIM2CNT &= NOT_BIT2; R_TIM2CNT |= BIT2; }
#define M_TIM2CNT_COUNTUP_TIMING_DISABLE     R_TIM2CNT &= NOT_BIT2;
#define F_TIM2CNT_COUNTUP_TIMING_ENABLED     ((R_TIM2CNT &= NOT_BIT2) == NOT_BIT2)
#define F_TIM2CNT_IRQ_ENABLED                ((R_TIM2CNT & BIT6) == BIT6)
#define F_TIM2CNT_TIMER_STARTED              ((R_TIM2CNT & BIT7) == BIT7)
#define M_TIM3CNT_COUNTUP_TIMING_ENABLE      { R_TIM3CNT &= NOT_BIT2; R_TIM3CNT |= BIT2; }
#define M_TIM3CNT_COUNTUP_TIMING_DISABLE     R_TIM3CNT &= NOT_BIT2;
#define F_TIM3CNT_COUNTUP_TIMING_ENABLED     ((R_TIM3CNT &= NOT_BIT2) == NOT_BIT2)
#define F_TIM3CNT_IRQ_ENABLED                ((R_TIM3CNT & BIT6) == BIT6)
#define F_TIM3CNT_TIMER_STARTED              ((R_TIM3CNT & BIT7) == BIT7)


//
// ---[ Color Conversion macros ]-------------------------------------------------------------------
//
#define RGB_GET_R(rgb)                      ((rgb) & 0x1f)
#define RGB_GET_G(rgb)                      (((rgb) >> 5) & 0x1f)
#define RGB_GET_B(rgb)                      (((rgb) >> 10) & 0x1f)


//
// ---[ Helper macros ]-----------------------------------------------------------------------------
//

//
// Background creation macro for gfx2gba name conventions.
// For example:
//             H_CREATE_BG(Background, 240, 160, 1, 0, 2, 0);
//
// Example creates a 240x160 pixel background on layer1, using 16 colors
// and prio2, no rotation map.
// Arrays are called Background_Tiles and Background_Map
//
#define H_BG_CREATE(ArrayName, Width, Height, BgNo, ColMode, Prio, RotationMap) \
    {                                                                           \
        u8 MapSize=0;                                                           \
        map_fragment_info *pMapInfo=NULL;                                       \
                                                                                \
        if(!RotationMap)                                                        \
        {                                                                       \
            if(Width <= 256 && Height <= 256)       MapSize=0;                  \
            else if(Width <= 512  && Height <= 256) MapSize=1;                  \
            else if(Width <= 256 && Height <= 512)  MapSize=2;                  \
            else if(Width <= 512 && Height <= 512)  MapSize=3;                  \
        }                                                                       \
        else                                                                    \
        {                                                                       \
            if(Width <= 128 && Height <= 128)        MapSize=0;                 \
            else if(Width <= 256  && Height <= 256)  MapSize=1;                 \
            else if(Width <= 512  && Height <= 512)  MapSize=2;                 \
            else if(Width <= 1024 && Height <= 1024) MapSize=3;                 \
        }                                                                       \
                                                                                \
        ham_bg[BgNo].ti = ham_InitTileSet((void*)ArrayName##_Tiles,             \
                            SIZEOF_16BIT(ArrayName##_Tiles), ColMode, 1);       \
                                                                                \
        ham_bg[BgNo].mi = ham_InitMapEmptySet(MapSize, RotationMap);            \
                                                                                \
        pMapInfo = ham_InitMapFragment((void*)ArrayName##_Map, Width/8,         \
                        Height/8,  0, 0, Width/8, Height/8, RotationMap);       \
                                                                                \
        ham_InsertMapFragment(pMapInfo, BgNo, 0, 0);                            \
        ham_InitBg(BgNo, FALSE, Prio, FALSE);                                   \
        pMapInfo = NULL;                                                        \
    }


#define H_BG_DEINIT_SELECT(bg0, bg1, bg2, bg3)                                  \
    {                                                                           \
        if(bg0) { ham_SetBgXY(0, 0, 0); ham_DeInitBg(0); }                      \
        if(bg1) { ham_SetBgXY(1, 0, 0); ham_DeInitBg(1); }                      \
        if(bg2) { ham_SetBgXY(2, 0, 0); ham_DeInitBg(2); }                      \
        if(bg3) { ham_SetBgXY(3, 0, 0); ham_DeInitBg(3); }                      \
    }


#define H_BG_VISIBLE_SELECT(bg0, bg1, bg2, bg3)                                 \
    {                                                                           \
        if(bg0) ham_SetBgVisible(0, TRUE);                                      \
        if(bg1) ham_SetBgVisible(1, TRUE);                                      \
        if(bg2) ham_SetBgVisible(2, TRUE);                                      \
        if(bg3) ham_SetBgVisible(3, TRUE);                                      \
    }


#define H_BG_INVISIBLE_SELECT(bg0, bg1, bg2, bg3)                               \
    {                                                                           \
        if(bg0) ham_SetBgVisible(0, FALSE);                                     \
        if(bg1) ham_SetBgVisible(1, FALSE);                                     \
        if(bg2) ham_SetBgVisible(2, FALSE);                                     \
        if(bg3) ham_SetBgVisible(3, FALSE);                                     \
    }


#define H_FULL_DARKEN                                                           \
    {                                                                           \
        ham_SetFxMode(FX_LAYER_SELECT(1,1,1,1,1,1),                             \
                      FX_LAYER_SELECT(1,1,1,1,1,1),                             \
                      FX_MODE_DARKEN);                                          \
                                                                                \
        ham_SetFxBrightnessLevel(16);                                           \
    }


#define H_WAIT_VSYNC { while((volatile u16)REG_VCOUNT != 160){} }




#ifdef __cplusplus
   extern "C" {
#endif

//
// ---[ Forward declarations ]----------------------------------------------------------------------
//
struct _TMapScrollInfo; 
struct _TMapDrawInfo;

//
// ---[ Map functions ]-----------------------------------------------------------------------------
//
#define BG_VIRTUAL   0x10

#define HEL_MAP_TILESX    31
#define HEL_MAP_TILESY    21

typedef void (*PMapNotifyFunc)(struct _TMapScrollInfo *pMapInfo, u32 X, u32 Y);
typedef void (*PMapDrawFunc)(struct _TMapScrollInfo *pMapInfo, struct _TMapDrawInfo *pMapColumnDrawInfo);

// void hel_MapDefaultColumnDrawer(TMapScrollInfo *pMapInfo, TMapColumnDrawInfo *pMapColumnDrawInfo)

typedef struct
{
   PMapNotifyFunc pOnTopRowChanged;
   PMapNotifyFunc pOnLeftColumnChanged;
}TMapNotifier, *PMapNotifier;

typedef struct _TMapParallaxInfo
{
   TPoint32 Ratio;                  // The Parallax-Ratio we use to Multiply the ScrollValue with
   TPoint32 CurrentAmount;          // Current amount, distance, since the last move
}TMapParallaxInfo, *PMapParallaxInfo;

typedef struct _TMapDrawInfo
{
  void *pSourceCurrentDataAddr;     // Pointer to source element in map
  u32 TargetColumn;                 // Horizontal target position in vram
  u32 TargetRow;                    // Vertical target position in vram
  u32 TargetScreenBlockBase;        // Address of target ScreenBlockBase
  u32 TargetCharacterBlockBase;     // Address of target CharacterBlockBase
  u32 Quantity;                     // Quantity of mapelements to update (in one direction)
  u32 Flags;                        // Flags
}TMapDrawInfo, *PMapDrawInfo;

typedef struct _TMapScrollInfo
{
   u8 BgNo;                         // The Backgroundnumber this structure is assigned to
   TPoint16 Size;                   // The size of the map in tiles
   void *pData;                     // Pointer to mapdata
   TPoint16 Position;               // The current scrollposition in tiles
   TPoint8 Runner;                  // The current scrollposition between current and next tile, in pixel
   u8 BoundsCheck;                  // Flag to enable/disable boundschecking
   TMapNotifier MapRedrawNotifiers; // Callback functions
   u8 UseParallax;                  // Flag to enable/disable Parallax-Scrolling
   TMapParallaxInfo ParallaxInfo;   // Additional information for Parallax-Scrolling
   u8 DataTypeSize;                 // The datatypesize of the data assigned to a VirtualMap
   void *pCustomData;               // A pointer to whatever data the user want to point to
   u8 ScrollFlags;                  // Flags which specify in what direction the system is allowed to scroll
   PMapDrawFunc pColumnDrawer;      // Functionpointer for column-map-drawer
   PMapDrawFunc pRowDrawer;         // Functionpointer for row-map-drawer
}TMapScrollInfo, *PMapScrollInfo;


#define __hel_MapInit(BgNo, TilesX, TilesY, Rotation, pMapData)   hel_MapInitEx(BgNo, TilesX, TilesY, Rotation, (Rotation)?MAP_SIZE_ROT_32X32:MAP_SIZE_32X32, pMapData)
#define __hel_MapSetBoundsCheck(pMapInfo, Value)                  (pMapInfo)->BoundsCheck = (Value);
#define __hel_MapIsBoundsCheckEnabled(pMapInfo)                   (pMapInfo)->BoundsCheck
#define __hel_MapSetPosition(pMapInfo, X, Y)                      hel_MapSetPositionInPixel(pMapInfo, (X)<<3, (Y)<<3);
#define __hel_MapSetParallax(pMapInfo, Value)                     (pMapInfo)->UseParallax = (Value);
#define __hel_MapIsParallaxEnabled(pMapInfo)                      (pMapInfo)->UseParallax
#define __hel_MapSetCustomData(pMapInfo, CustomData)              (pMapInfo)->pCustomData = (void*)(CustomData);
#define __hel_MapGetCustomData(pMapInfo)                          ((void*)(pMapInfo->pCustomData))
#define __hel_MapSetScrollFlags(pMapInfo, Flags)                  (pMapInfo)->ScrollFlags = Flags;
#define __hel_MapGetScrollFlags(pMapInfo)                         (pMapInfo)->ScrollFlags

#ifdef HAM_DEBUGGER
  extern TMapScrollInfo hel_MapInit(u8 BgNo, u16 TilesX, u16 TilesY, u8 Rotation, void *pMapData);
  extern void hel_MapSetBoundsCheck(TMapScrollInfo *pMapInfo, u8 Value);
  extern u8 hel_MapIsBoundsCheckEnabled(TMapScrollInfo *pMapInfo);
  extern void hel_MapSetPosition(TMapScrollInfo *pMapInfo, u32 X, u32 Y);
  extern void hel_MapSetParallax(TMapScrollInfo *pMapInfo, u8 Value);
  extern u8 hel_MapIsParallaxEnabled(TMapScrollInfo *pMapInfo);
  extern void hel_MapSetCustomData(TMapScrollInfo *pMapInfo, void *pCustomData);
  extern void *hel_MapGetCustomData(TMapScrollInfo *pMapInfo);
  extern void hel_MapSetScrollFlags(TMapScrollInfo *pMapInfo, u32 Flags);
  extern u8 hel_MapGetScrollFlags(TMapScrollInfo *pMapInfo);
#else
  #define hel_MapInit(BgNo, TilesX, TilesY, Rotation, pMapData)   __hel_MapInit(BgNo, TilesX, TilesY, Rotation, pMapData)
  #define hel_MapSetBoundsCheck(pMapInfo, Value)                  __hel_MapSetBoundsCheck(pMapInfo, Value)
  #define hel_MapIsBoundsCheckEnabled(pMapInfo)                   __hel_MapIsBoundsCheckEnabled(pMapInfo)
  #define hel_MapSetPosition(pMapInfo, X, Y)                      __hel_MapSetPosition(pMapInfo, X, Y)
  #define hel_MapSetParallax(pMapInfo, Value)                     __hel_MapSetParallax(pMapInfo, Value)
  #define hel_MapIsParallaxEnabled(pMapInfo)                      __hel_MapIsParallaxEnabled(pMapInfo)
  #define hel_MapSetCustomData(pMapInfo, pCustomData)             __hel_MapSetCustomData(pMapInfo, pCustomData)
  #define hel_MapGetCustomData(pMapInfo)                          __hel_MapGetCustomData(pMapInfo)
  #define hel_MapSetScrollFlags(pMapInfo, Flags)                  __hel_MapSetScrollFlags(pMapInfo, Flags)
  #define hel_MapGetScrollFlags(pMapInfo)                         __hel_MapGetScrollFlags(pMapInfo)
#endif


extern TMapScrollInfo hel_MapInitVirtual(u16 TilesX, u16 TilesY, u8 DataTypeSize, void *pMapData);
extern void hel_MapDeInit(TMapScrollInfo *pMapInfo);
extern TMapScrollInfo hel_MapInitEx(u8 BgNo, u16 TilesX, u16 TilesY, u8 Rotation, u8 MapSize, void *pMapData);
extern void hel_MapSetCallbacks(TMapScrollInfo *pMapInfo, PMapNotifyFunc pOnTopRowChanged, PMapNotifyFunc pOnLeftColumnChanged);
extern u8 hel_MapScrollBy(TMapScrollInfo *pMapInfo, s32 DeltaX, s32 DeltaY);
extern void hel_MapSetPositionInPixel(TMapScrollInfo *pMapInfo, u32 X, u32 Y);
extern TPoint32 hel_MapGetPosition(TMapScrollInfo *pMapInfo);
extern TPoint32 hel_MapGetPositionInPixel(TMapScrollInfo *pMapInfo);
extern void hel_MapRedraw(TMapScrollInfo *pMapInfo);
extern u32 hel_MapBatchScrollBy(TMapScrollInfo *pMapInfo, s32 DeltaX, s32 DeltaY, int Count);
extern void hel_MapSetParallaxRatio(TMapScrollInfo *pMapInfo, FIXED RatioX, FIXED RatioY);
extern u16 hel_MapGetTileAt(TMapScrollInfo *pMapInfo, u32 ScreenX, u32 ScreenY);
extern void *hel_MapGetTilePtrAt(TMapScrollInfo *pMapInfo, u32 ScreenX, u32 ScreenY);
extern TPoint32 hel_MapGetPositionInPixelFrom(TMapScrollInfo *pMapInfo, u32 TileX, u32 TileY);
extern u8 hel_MapBatchScrollByEx(TMapScrollInfo *pMapInfos, s32 DeltaX, s32 DeltaY, int Count, u32 PrimaryLayer);
extern void hel_MapTransmitPosition(TMapScrollInfo *pMapInfo);
extern void hel_MapJumpTo(TMapScrollInfo *pMapInfo, u32 JumpToFlag);
extern void hel_MapSetDynamicTileReloading(TMapScrollInfo *pMapInfo, u32 Enable);


//
// ---[ Map Drawer functions ]----------------------------------------------------------------------
//

extern const u32 HEL_SETMAPTILE_LUT_X[];
extern const u32 HEL_SETMAPTILE_LUT_Y[];

extern void hel_MapDrawerStandardColumnFuncImplementation(TMapScrollInfo *pMapInfo, TMapDrawInfo *pDrawInfo);
extern void hel_MapDrawerStandardColumnRotationFuncImplementation(TMapScrollInfo *pMapInfo, TMapDrawInfo *pDrawInfo);
extern void hel_MapDrawerStandardRowFuncImplementation(TMapScrollInfo *pMapInfo, TMapDrawInfo *pDrawInfo);
extern void hel_MapDrawerStandardRowRotationFuncImplementation(TMapScrollInfo *pMapInfo, TMapDrawInfo *pDrawInfo);
extern void hel_MapDrawerTileColumnFuncImplementation(TMapScrollInfo *pMapInfo, TMapDrawInfo *pDrawInfo) ATTR_FUNC_IN_IWRAM;
extern void hel_MapDrawerTileRowFuncImplementation(TMapScrollInfo *pMapInfo, TMapDrawInfo *pDrawInfo) ATTR_FUNC_IN_IWRAM;


//
// ---[ Map Camera functions ]----------------------------------------------------------------------
//

#define MAP_CAM_NEXTKEYPOINT    0x11
#define MAP_CAM_LASTKEYREACHED  0x12

typedef TPoint16 TMapCamKeyPoint;
typedef TMapCamKeyPoint *PMapCamKeyPoint;


typedef struct _TMapCamScrollInfo
{
   TMapCamKeyPoint *pKeyPoints;
   u16 nKeyPoints;
   u16 CurKeyPoint;
}TMapCamScrollInfo, *PMapCamScrollInfo;

extern u8 hel_MapScrollTo(TMapScrollInfo *pMapInfo, TMapCamScrollInfo *pMapCamInfo, u32 SpeedX, u32 SpeedY);



//
// ---[ Tile functions ]----------------------------------------------------------------------------
//

typedef struct TTileInfo
{
  u8 *pTileData;    // Pointer to source tiledata, usually located in ROM
  u16 NumTilesRom;  // Number of tiles in source tiledata
  u16 *pRomToRam;   // Pointer to an allocated buffer of NumTilesRom 16bit words (should be located in EWRAM)
  u16 NumTilesRam;  // Number of tiles to store in VRAM and work with
  u16 *pRamToRom;   // Pointer to an allocated buffer of NumTilesRam 16bit words (should be located in EWRAM)
  u16 *pRefCount;   // Pointer to an allocated buffer of NumTilesRam 16bit words (should be located in EWRAM)
  u16 *pTileSlot;   // Pointer to an allocated buffer of NumTilesRam 16bit words (should be located in EWRAM)
  u16 SlotCounter;  // Index to the next free slot in pRamToRom
  tile_info *ti;    // Needed by HAM, for tileset managing
  u8  RefCount;     // Reference Counter. Specifies how often is it used/shared.
}TTileInfo, *PTileInfo;

typedef struct TTileSystemInfo
{
  u16 BgNo;             // Background the tilesystem is assigned to
  u16 PalNo;            // Palettenumber if it's a 16 color background, otherwise set it to zero
  TTileInfo *pTileInfo; // TileInfo
}TTileSystemInfo, *PTileSystemInfo;

#define TILE_UNUSED 0xffff
#define __hel_TileIsGraphicLoaded(BgNo, RomTileNo)                    (TILE_UNUSED != hel_TileInfo[BgNo].pRomToRam[RomTileNo])
#define __hel_TileReloadGraphic16(BgNo, TargetRomTileNo, pSource)     __hel_DmaCopy16(MEM_CHR_BB(hel_TileSystemInfo[BgNo].pTileInfo->ti->cbb) + (hel_TileSystemInfo[BgNo].pTileInfo->pRomToRam[TargetRomTileNo]<<5), pSource, 16)
#define __hel_TileReloadGraphic256(BgNo, TargetRomTileNo, pSource)    __hel_DmaCopy16(MEM_CHR_BB(hel_TileSystemInfo[BgNo].pTileInfo->ti->cbb) + (hel_TileSystemInfo[BgNo].pTileInfo->pRomToRam[TargetRomTileNo]<<6), pSource, 32)


#ifdef HAM_DEBUGGER
  extern u32 hel_TileIsGraphicLoaded(u32 BgNo, u32 RomTileNo);
  extern void hel_TileReloadGraphic16(u32 BgNo, u32 TargetRomTileNo, const u8* pSource);
  extern void hel_TileReloadGraphic256(u32 BgNo, u32 TargetRomTileNo, const u8* pSource);
#else
  #define hel_TileIsGraphicLoaded(BgNo, RomTileNo)                    __hel_TileIsGraphicLoaded(BgNo, RomTileNo)
  #define hel_TileReloadGraphic16(BgNo, TargetRomTileNo, pSource)     __hel_TileReloadGraphic16(BgNo, TargetRomTileNo, pSource)
  #define hel_TileReloadGraphic256(BgNo, TargetRomTileNo, pSource)    __hel_TileReloadGraphic256(BgNo, TargetRomTileNo, pSource)
#endif

extern void hel_TileInit(u32 BgNo, const u8 *pTileData, u16 NumTilesRom, u16 *pBufferA, u16 NumTilesRam, u16 *pBufferB, u8 ColMode, u8 PalNo, u8 CbbOnlyMode);
extern void hel_TileDeInit(u32 BgNo);
extern void hel_TileShare(u8 TargetBgNo, u8 SourceBgNo, u8 PalNo);
extern void hel_TileReloadGraphic(u32 BgNo, u32 TargetRomTileNo, const u8* pSource);
extern void hel_TilePreloadGraphic(u32 BgNo, u32 RomTileNo);
extern void hel_TileReleaseGraphic(u32 BgNo, u32 RomTileNo);

// Internal use only!
extern TTileInfo hel_TileInfo[];
extern TTileSystemInfo hel_TileSystemInfo[];
extern void ATTR_FUNC_IN_IWRAM hel_TileSetMapTile(u32 BgNo, u16 *pDest, u16 *pSource, u32 Flags);


//
// ---[ Text functions ]----------------------------------------------------------------------------
//

typedef struct
{
   u8 BgNo;             // What BackgroundNumber to use
   char *CharOrder;     // Order of character arrangement on texture
   u8 CharIndex[256];   // Private use only
   u8 ColMode;          // What Colormode to use, 0=16 Color, 1=256 Colors
   u8 PalNo;            // Palettebank-Number, only valid in 16 Colormode
}TCustomTextInfo, *PCustomTextInfo;

extern void hel_CustomTextInit(TCustomTextInfo *pTextInfo, void *pSrcTiles, u16 SizeU16, u8 ColMode, u8 PalNo, u8 MapSize, u8 MapRot);
extern void hel_CustomTextClear(TCustomTextInfo *pTextInfo);
extern void hel_CustomTextClearRow(TCustomTextInfo *pTextInfo, u32 y);
extern void hel_CustomTextPrint(TCustomTextInfo *pTextInfo, u32 x, u32 y, char *pText);
extern void hel_CustomTextPrintFmt(TCustomTextInfo *pTextInfo, u32 x, u32 y, char *pFmt, ...);
extern u8 hel_CustomTextType(TCustomTextInfo *pTextInfo, u32 x, u32 y, u8 Delay, u16 Wait, char *pText);


//
// ---[ Pad functions ]-----------------------------------------------------------------------------
//
#define PAD_BEHAVIOUR_PRESSED_ON_BUTTON_UP      0
#define PAD_BEHAVIOUR_PRESSED_ON_BUTTON_DOWN    1

typedef struct _TPadButtons
{
   u8 Left:1;   // -Left-
   u8 Right:1;  // -Right-
   u8 Down:1;   // -Down-
   u8 Up:1;     // -Up-
   u8 Start:1;  // -Start-
   u8 Select:1; // -Select-
   u8 A:1;      // -A-
   u8 B:1;      // -B-
   u8 L:1;      // -L-, Shoulderbutton
   u8 R:1;      // -R-, Shoulderbutton
   u8 None:1;   // If this member is 0, no button is pressed
}TPadButtons, *PPadButtons;

typedef struct
{
   TPadButtons Pressed; // Holds information about what buttons have been pressed, since last #hel_PadReadState call
   TPadButtons Held;    // Holds information about what buttons are held down, since last #hel_PadReadState call
   u8 Behaviour;        // Specifies the pad behaviour, if you want the Pressed-State on button push or release
}TPad, *PPad;

// Global Pad-Variable which holds information about
// the current padstate, once #hel_PadReadState has
// been called.
extern TPad hel_Pad; // Pad variable

extern void hel_PadInit(void);
extern void hel_PadReset(void);
extern void hel_PadReadState(void);
extern void hel_PadSetBehaviour(u32 NewBehaviour);

//
// ---[ Palette functions ]-------------------------------------------------------------------------
//
#define __hel_PalBgInvert16(PaletteBank)                        hel_PalBgInvertEx((PaletteBank)<<4, ((PaletteBank)<<4)+16)
#define __hel_PalBgInvert256()                                  hel_PalBgInvertEx(0, 0xff)
#define __hel_PalBgClear16(PaletteBank)                         hel_DmaSet16((void*)MEM_PAL_BG_16(PaletteBank), 0, 16)
#define __hel_PalBgClear256()                                   hel_DmaSet16((void*)MEM_PAL_BG, 0, 256)
#define __hel_PalBgClearEx(StartIndex, EndIndex, R, G, B)       hel_DmaSet16((void*)(MEM_PAL_BG+((StartIndex)<<1)), RGB(R,G,B), ((EndIndex)-(StartIndex)))
#define __hel_PalBgLoad16(SourceData, PaletteBank)              hel_DmaCopy16((void*)MEM_PAL_BG_16(PaletteBank), (void*)(SourceData), 16)
#define __hel_PalBgSave16(pDest, PaletteBank)                   hel_DmaCopy16((void*)(pDest), (void*)MEM_PAL_BG_16(PaletteBank), 16)
#define __hel_PalBgLoad256(SourceData)                          hel_DmaCopy16((void*)MEM_PAL_BG, (void*)(SourceData), 256)
#define __hel_PalBgSave256(pDest)                               hel_DmaCopy16((void*)(pDest), (void*)MEM_PAL_BG, 256)
#define __hel_PalObjLoad16(SourceData, PaletteBank)             hel_DmaCopy16((void*)MEM_PAL_OBJ_16(PaletteBank), (void*)(SourceData), 16)
#define __hel_PalObjSave16(pDest, PaletteBank)                  hel_DmaCopy16((void*)(pDest), (void*)MEM_PAL_OBJ_16(PaletteBank), 16)
#define __hel_PalObjLoad256(SourceData)                         hel_DmaCopy16((void*)MEM_PAL_OBJ, (void*)(SourceData), 256)
#define __hel_PalObjSave256(pDest)                              hel_DmaCopy16((void*)(pDest), (void*)MEM_PAL_OBJ, 256)
#define __hel_PalBgLoad(SourceData, NumColors)                  hel_DmaCopy16((void*)MEM_PAL_BG, (void*)(SourceData), NumColors)
#define __hel_PalObjLoad(SourceData, NumColors)                 hel_DmaCopy16((void*)MEM_PAL_OBJ, (void*)(SourceData), NumColors)
#define __hel_PalBgLoad256UnComp(SourceData, CompressionType)   hel_SwiUnComp((void*)MEM_PAL_BG, SourceData, CompressionType)
#define __hel_PalBgLoad16UnComp(SourceData, PaletteBank, CompressionType)    hel_SwiUnComp((void*)MEM_PAL_BG_16(PaletteBank), SourceData, CompressionType)
#define __hel_PalObjLoad256UnComp(SourceData, CompressionType)  hel_SwiUnComp((void*)MEM_PAL_OBJ, SourceData, CompressionType)
#define __hel_PalObjLoad16UnComp(SourceData, PaletteBank, CompressionType)   hel_SwiUnComp((void*)MEM_PAL_OBJ_16(PaletteBank), SourceData, CompressionType)


#ifdef HAM_DEBUGGER
  extern void hel_PalBgInvert16(u32 PaletteBank);
  extern void hel_PalBgInvert256(void);
  extern void hel_PalBgClear16(u32 PaletteBank);
  extern void hel_PalBgClear256(void);
  extern void hel_PalBgClearEx(u32 StartIndex, u32 EndIndex, u32 Red, u32 Green, u32 Blue);
  extern void hel_PalBgLoad16(void *SourceData, u32 PaletteBank);
  extern void hel_PalBgSave16(void *pDest, u32 PaletteBank);
  extern void hel_PalBgLoad256(void *SourceData);
  extern void hel_PalBgSave256(void *pDest);
  extern void hel_PalObjLoad16(void *SourceData, u32 PaletteBank);
  extern void hel_PalObjSave16(void *pDest, u32 PaletteBank);
  extern void hel_PalObjLoad256(void *SourceData);
  extern void hel_PalObjSave256(void *pDest);
  extern void hel_PalBgLoad(void *SourceData, u32 NumColors);
  extern void hel_PalObjLoad(void *SourceData, u32 NumColors);
  extern void hel_PalBgLoad256UnComp(void *SourceData, u32 CompressionType);
  extern void hel_PalBgLoad16UnComp(void *SourceData, u32 PaletteBank, u32 CompressionType);
  extern void hel_PalObjLoad256UnComp(void *SourceData, u32 CompressionType);
  extern void hel_PalObjLoad16UnComp(void *SourceData, u32 PaletteBank, u32 CompressionType);
#else
  #define hel_PalBgInvert16(PaletteBank)                        __hel_PalBgInvert16(PaletteBank)
  #define hel_PalBgInvert256()                                  __hel_PalBgInvert256()
  #define hel_PalBgClear16(PaletteBank)                         __hel_PalBgClear16(PaletteBank)
  #define hel_PalBgClear256()                                   __hel_PalBgClear256()
  #define hel_PalBgClearEx(StartIndex, EndIndex, R, G, B)       __hel_PalBgClearEx(StartIndex, EndIndex, R, G, B)
  #define hel_PalBgLoad16(SourceData, PaletteBank)              __hel_PalBgLoad16(SourceData, PaletteBank)
  #define hel_PalBgSave16(pDest, PaletteBank)                   __hel_PalBgSave16(pDest, PaletteBank)
  #define hel_PalBgLoad256(SourceData)                          __hel_PalBgLoad256(SourceData)
  #define hel_PalBgSave256(pDest)                               __hel_PalBgSave256(pDest)
  #define hel_PalObjLoad16(SourceData, PaletteBank)             __hel_PalObjLoad16(SourceData, PaletteBank)
  #define hel_PalObjSave16(pDest, PaletteBank)                  __hel_PalObjSave16(pDest, PaletteBank)
  #define hel_PalObjLoad256(SourceData)                         __hel_PalObjLoad256(SourceData)
  #define hel_PalObjSave256(pDest)                              __hel_PalObjSave256(pDest)
  #define hel_PalBgLoad(SourceData, NumColors)                  __hel_PalBgLoad(SourceData, NumColors)
  #define hel_PalObjLoad(SourceData, NumColors)                 __hel_PalObjLoad(SourceData, NumColors)
  #define hel_PalBgLoad256UnComp(SourceData, CompressionType)   __hel_PalBgLoad256UnComp(SourceData, CompressionType)
  #define hel_PalBgLoad16UnComp(SourceData, PaletteBank, CompressionType)    __hel_PalBgLoad16UnComp(SourceData, PaletteBank, CompressionType)
  #define hel_PalObjLoad256UnComp(SourceData, CompressionType)  __hel_PalObjLoad256UnComp(SourceData, CompressionType)
  #define hel_PalObjLoad16UnComp(SourceData, PaletteBank, CompressionType)   __hel_PalObjLoad16UnComp(SourceData, PaletteBank, CompressionType)
#endif


extern void hel_PalBgInvertEx(u32 StartIndex, u32 EndIndex);
extern void hel_PalBgInterpolate16(void *pPaletteA, void *pPaletteB, u32 PaletteBank, u32 Step);
extern void hel_PalBgInterpolate256(void *pPaletteA, void *pPaletteB, u32 Step);



//
// ---[ Obj functions ]-----------------------------------------------------------------------------
//

extern const u32 OBJ_SIZE_LUT[3][4] ATTR_ALIGNED(4);
extern const u32 HEL_OBJ_SHAPESIZE_TO_HALFWORD[2][4][4] ATTR_ALIGNED(4);
extern TPointU8 __hel_ObjMosaicSize;

#define __hel_ObjExists(ObjNo)                (ham_obj[ObjNo].taken == 1)
#define __hel_ObjIsHFlip(ObjNo)               ((ham_obj[ObjNo].objatr1 >> 12) & 1)
#define __hel_ObjIsVFlip(ObjNo)               ((ham_obj[ObjNo].objatr1 >> 13) & 1)
#define __hel_ObjIsVisible(ObjNo)             (ham_obj[ObjNo].active == TRUE)
#define __hel_ObjIsMosaic(ObjNo)              ((ham_obj[ObjNo].objatr0 >> 12) & 1)
#define __hel_ObjIsRotScale(ObjNo)            ((ham_obj[ObjNo].objatr0 >>  8) & 1)
#define __hel_ObjGetColorMode(ObjNo)          ((ham_obj[ObjNo].objatr0 >> 13) & 1)
#define __hel_ObjGetMode(ObjNo)               ((ham_obj[ObjNo].objatr0 >> 10) & 2)
#define __hel_ObjBringToFront(ObjNo)          ham_SetObjBefore(ObjNo, ham_obj_first)
#define __hel_ObjGetFirst()                   ham_obj_first
#define __hel_ObjHideAll()                    hel_ObjSetVisibleAll(FALSE)
#define __hel_ObjShowAll()                    hel_ObjSetVisibleAll(TRUE)
#define __hel_ObjToggleVisible(ObjNo)         ham_obj[ObjNo].active = !ham_obj[ObjNo].active
#define __hel_ObjSetVisible(ObjNo, Value)     ham_obj[ObjNo].active = (Value);
#define __hel_ObjSetX(ObjNo, X)               { ham_obj[ObjNo].objatr1 &= 0xFE00; ham_obj[ObjNo].objatr1 |= ((X) & 0x01FF); }
#define __hel_ObjSetY(ObjNo, Y)               { ham_obj[ObjNo].objatr0 &= 0xFF00; ham_obj[ObjNo].objatr0 |= ((Y) & 0x00FF); }
#define __hel_ObjSetXY(ObjNo, X, Y)           { __hel_ObjSetX(ObjNo, X); __hel_ObjSetY(ObjNo, Y); }
#define __hel_ObjSetHFlip(ObjNo, Value)       { ham_obj[ObjNo].objatr1 &= 0xEFFF; ham_obj[ObjNo].objatr1 |= ((Value) << 12); }
#define __hel_ObjSetVFlip(ObjNo, Value)       { ham_obj[ObjNo].objatr1 &= 0xDFFF; ham_obj[ObjNo].objatr1 |= ((Value) << 13); }
#define __hel_ObjSetPrio(ObjNo, Prio)         { ham_obj[ObjNo].objatr2 &= 0xF3FF; ham_obj[ObjNo].objatr2 |= ((Prio) << 10); }
#define __hel_ObjGetGfxSlot(ObjNo)            (ham_obj[ObjNo].objatr2 & 0x03ff)
#define __hel_ObjSetMode(ObjNo, Mode)         { ham_obj[ObjNo].objatr0 &= 0xf3ff; ham_obj[ObjNo].objatr0 |= ((Mode) << 10); }
#define __hel_ObjGetPrio(ObjNo)               ((ham_obj[ObjNo].objatr2 >> 10) & 3)
#define __hel_ObjSelectRotScale(ObjNo, RotNo) { ham_obj[ObjNo].objatr1 &= 0xC1FF; ham_obj[ObjNo].objatr1 |= (RotNo); }
#define __hel_ObjSetDblSize(ObjNo, Value)     { ham_obj[ObjNo].objatr0 &= 0xFDFF; ham_obj[ObjNo].objatr0 |= ((Value)<<9); }
#define __hel_ObjGetShape(ObjNo)              (ham_obj[ObjNo].objatr0>>14)
#define __hel_ObjGetSize(ObjNo)               (ham_obj[ObjNo].objatr1>>14)
#define __hel_ObjGetSizeU16(ObjNo)            HEL_OBJ_SHAPESIZE_TO_HALFWORD[__hel_ObjGetColorMode(ObjNo)][__hel_ObjGetSize(ObjNo)][__hel_ObjGetShape(ObjNo)]
#define __hel_ObjUpdateGfx(ObjNo, pSrc)       hel_DmaCopy16((void*)MEM_OBJ_SLOT_PTR(__hel_ObjGetGfxSlot(ObjNo)), (void*)pSrc, __hel_ObjGetSizeU16(ObjNo))
#define __hel_ObjUpdateGfxUnComp(ObjNo, pSrc, CompressionType) __hel_SwiUnComp((void*)MEM_OBJ_SLOT_PTR(__hel_ObjGetGfxSlot(ObjNo)), (void*)pSrc, CompressionType)



#ifdef HAM_DEBUGGER
    extern u8 hel_ObjExists(u8 ObjNo);
    extern u8 hel_ObjIsHFlip(u8 ObjNo);
    extern u8 hel_ObjIsVFlip(u8 ObjNo);
    extern u8 hel_ObjIsVisible(u8 ObjNo);
    extern u8 hel_ObjIsMosaic(u8 ObjNo);
    extern u8 hel_ObjIsRotScale(u8 ObjNo);
    extern u8 hel_ObjGetColorMode(u8 ObjNo);
    extern void hel_ObjBringToFront(u8 ObjNo);
    extern u8 hel_ObjGetFirst(void);
    extern void hel_ObjHideAll(void);
    extern void hel_ObjShowAll(void);
    extern u8 hel_ObjToggleVisible(u8 ObjNo);
    extern void hel_ObjSetVisible(u8 ObjNo, u8 Value);
    extern void hel_ObjSetX(u8 ObjNo, s16 X);
    extern void hel_ObjSetY(u8 ObjNo, s16 X);
    extern void hel_ObjSetXY(u8 ObjNo, s16 X, s16 Y);
    extern void hel_ObjSetHFlip(u8 ObjNo, u8 Value);
    extern void hel_ObjSetVFlip(u8 ObjNo, u8 Value);
    extern void hel_ObjSetPrio(u8 ObjNo, u32 Prio);
    extern u16 hel_ObjGetGfxSlot(u8 ObjNo);
    extern void hel_ObjSetMode(u8 ObjNo, u8 Mode);
    extern u8 hel_ObjGetPrio(u8 ObjNo);
    extern void hel_ObjSelectRotScale(u8 ObjNo, u8 RotNo);
    extern void hel_ObjSetDblSize(u8 ObjNo, u8 Value);
    extern u32 hel_ObjGetShape(u8 ObjNo);
    extern u32 hel_ObjGetSize(u8 ObjNo);
    extern u32 hel_ObjGetSizeU16(u8 ObjNo);
    extern void hel_ObjUpdateGfx(u8 ObjNo, void* pSrc);
    extern void hel_ObjUpdateGfxUnComp(u8 ObjNo, void* pSrc, u32 CompressionType);
#else
    #define hel_ObjExists(ObjNo)                __hel_ObjExists(ObjNo)
    #define hel_ObjIsHFlip(ObjNo)               __hel_ObjIsHFlip(ObjNo)
    #define hel_ObjIsVFlip(ObjNo)               __hel_ObjIsVFlip(ObjNo)
    #define hel_ObjIsVisible(ObjNo)             __hel_ObjIsVisible(ObjNo)
    #define hel_ObjIsMosaic(ObjNo)              __hel_ObjIsMosaic(ObjNo)
    #define hel_ObjIsRotScale(ObjNo)            __hel_ObjIsRotScale(ObjNo)
    #define hel_ObjGetColorMode(ObjNo)          __hel_ObjGetColorMode(ObjNo)
    #define hel_ObjGetMode(ObjNo)               __hel_ObjGetMode(ObjNo)
    #define hel_ObjBringToFront(ObjNo)          __hel_ObjBringToFront(ObjNo)
    #define hel_ObjGetFirst()                   __hel_ObjGetFirst()
    #define hel_ObjHideAll()                    __hel_ObjHideAll()
    #define hel_ObjShowAll()                    __hel_ObjShowAll()
    #define hel_ObjToggleVisible(ObjNo)         __hel_ObjToggleVisible(ObjNo)
    #define hel_ObjSetVisible(ObjNo, Value)     __hel_ObjSetVisible(ObjNo, Value)
    #define hel_ObjSetX(ObjNo, X)               __hel_ObjSetX(ObjNo, X)
    #define hel_ObjSetY(ObjNo, Y)               __hel_ObjSetY(ObjNo, Y)
    #define hel_ObjSetXY(ObjNo, X, Y)           __hel_ObjSetXY(ObjNo, X, Y)
    #define hel_ObjSetHFlip(ObjNo, Value)       __hel_ObjSetHFlip(ObjNo, Value)
    #define hel_ObjSetVFlip(ObjNo, Value)       __hel_ObjSetHFlip(ObjNo, Value)
    #define hel_ObjSetPrio(ObjNo, Prio)         __hel_ObjSetPrio(ObjNo, Prio)
    #define hel_ObjGetGfxSlot(ObjNo)            __hel_ObjGetGfxSlot(ObjNo)
    #define hel_ObjSetMode(ObjNo, Mode)         __hel_ObjSetMode(ObjNo, Mode)
    #define hel_ObjGetPrio(ObjNo)               __hel_ObjGetPrio(ObjNo)
    #define hel_ObjSelectRotScale(ObjNo, RotNo) __hel_ObjSelectRotScale(ObjNo, RotNo)
    #define hel_ObjSetDblSize(ObjNo, Value)     __hel_ObjSetDblSize(ObjNo, Value)
    #define hel_ObjGetShape(ObjNo)              __hel_ObjGetShape(ObjNo)
    #define hel_ObjGetSize(ObjNo)               __hel_ObjGetSize(ObjNo)
    #define hel_ObjGetSizeU16(ObjNo)            __hel_ObjGetSizeU16(ObjNo)
    #define hel_ObjUpdateGfx(ObjNo, pSrc)       __hel_ObjUpdateGfx(ObjNo, pSrc)
    #define hel_ObjUpdateGfxUnComp(ObjNo, pSrc, CompressionType)    __hel_ObjUpdateGfxUnComp(ObjNo, pSrc, CompressionType)
#endif  // HAM_DEBUGGER


#define hel_ObjCalcFrameOffset(ObjNo, Index) (OBJ_SIZE_LUT[ham_obj[ObjNo].objatr0 >> 14][ham_obj[ObjNo].objatr1 >> 14] * Index) >> (!((ham_obj[ObjNo].objatr0 >> 13)&1))


extern u8 hel_ObjCreate(void* pSrc, u16 ObjShape, u16 ObjSize, u16 ObjMode, u16 ColMode, u16 PalNo, u16 Mosaic, u16 HFlip, u16 VFlip, u16 Prio, u16 DblSize, s16 X, s16 Y);
extern u8 hel_ObjCreate256(void* pSrc, u16 ObjShape, u16 ObjSize, u16 ObjMode, s16 X, s16 Y);
extern u8 hel_ObjCreate16(void* pSrc, u16 ObjShape, u16 ObjSize, u16 ObjMode, u16 PalNo, s16 X, s16 Y);
extern void ATTR_DEPRECATED hel_ObjUpdateInOAM(u8 ObjNo);
extern void hel_ObjSetVisibleAll(u8 Show);
extern u32 hel_ObjCountVisible(void);
extern inline void hel_ObjSetMosaicSize(u8 HSize, u8 VSize);
extern void hel_ObjSendToBack(u32 ObjNo);
extern void hel_ObjClearOAM(void);

// test-stuff
#ifdef HAM_DEBUGGER
  extern u8 __hel_ObjRotScaleRefCnt[];
#endif

extern void hel_ObjRotScaleInit(u32 RotNo, u32 Angle, s32 ZoomX, s32 ZoomY);
extern void hel_ObjRotScaleDeInit(u32 RotNo);
extern void hel_ObjRotScaleUpdate(u32 RotNo, u32 Angle, s32 ZoomX, s32 ZoomY);


//
// ---[ BG functions ]------------------------------------------------------------------------------
//

extern TPointU8 __hel_BgMosaicSize;

extern inline void hel_BgSetMosaic(u8 BgNo, u8 Value);
extern inline void hel_BgSetMosaicSize(u8 HSize, u8 VSize);


//
// ---[ DMA functions ]-----------------------------------------------------------------------------
//

#define __hel_DmaCopy16(pDest, pSrc, SizeIn16BitWords)      __hel_DmaCopy16Implementation((void*)(pSrc), (void*)(pDest), SizeIn16BitWords)
#define __hel_DmaCopy32(pDest, pSrc, SizeIn32BitWords)      __hel_DmaCopy32Implementation((void*)(pSrc), (void*)(pDest), SizeIn32BitWords)
#define __hel_DmaSet16(pDest, SrcValue, SizeIn16BitWords)   __hel_DmaSet16Implementation(SrcValue, (void*)(pDest), SizeIn16BitWords)
#define __hel_DmaZeroMemory(pDest, SizeInBytes)             hel_DmaSet8((void*)(pDest), 0, SizeInBytes)


#ifdef HAM_DEBUGGER
  extern void hel_DmaCopy16(void *pDest, const void *pSrc, u32 SizeIn16BitWords);
  extern void hel_DmaCopy32(void *pDest, const void *pSrc, u32 SizeIn32BitWords);
  extern void hel_DmaSet16(void *pDest, u16 SrcValue, u32 SizeIn16BitWords);
  extern void hel_DmaZeroMemory(void *pDest, u32 SizeInBytes);
#else
  #define hel_DmaCopy16(pDest, pSrc, SizeIn16BitWords)    __hel_DmaCopy16(pDest, pSrc, SizeIn16BitWords)
  #define hel_DmaCopy32(pDest, pSrc, SizeIn16BitWords)    __hel_DmaCopy32(pDest, pSrc, SizeIn16BitWords)
  #define hel_DmaSet16(pDest, SrcValue, SizeIn16BitWords) __hel_DmaSet16(pDest, SrcValue, SizeIn16BitWords)
  #define hel_DmaZeroMemory(pDest, SizeInBytes)           __hel_DmaZeroMemory(pDest, SizeInBytes)
#endif 


// helDma.c
extern void hel_DmaSet8(void *pDest, u8 SrcValue, u32 SizeInBytes);
extern void hel_DmaSet32(void *pDest, u32 SrcValue, u32 SizeIn32BitWords);

// helDmaImplementation.s
extern void __hel_DmaCopy16Implementation(const void *pSrc, void *pDest, u32 SizeIn16BitWords);
extern void __hel_DmaCopy32Implementation(const void *pSrc, void *pDest, u32 SizeIn32BitWords);
extern void __hel_DmaSet16Implementation(u16 SrcValue, void *pDest, u32 SizeIn16BitWords);

//
// ---[ Sprite functions ]--------------------------------------------------------------------------
//

typedef struct _TSprite
{
   u8 ObjNo;
   s16 X,Y;
}TSprite, *PSprite;


//
// ---[ Splash functions ]--------------------------------------------------------------------------
//
extern void hel_Splash(void *pBitmap, void *pPalette, u8 SplashTypeIn, u8 SplashTypeOut, u8 FadeInSpeed, u8 FadeOutSpeed, u8 WaitFrames, u8 AfterWaitFrames);
extern void hel_SplashUnComp(void *pBitmap, void *pPalette, u8 SplashTypeIn, u8 SplashTypeOut, u8 FadeInSpeed, u8 FadeOutSpeed, u8 WaitFrames, u8 AfterWaitFrames, u32 CompressionType);


//
// ---[ Fade functions ]----------------------------------------------------------------------------
//

typedef struct _TFadeInfo
{
   u16 *pPalette[16];     // Pointer to Source-Palette(s)
   u8 PalMode;            // Palettemode, Background or Object Palette
   u8 ColMode;            // Colormode, 16 or 256 Colors
   u8 PalNo;              // Palettenumber of first Palette to fade (OBSOLETE)
   u8 Palettes;           // Which Palettes need to fade (each bit represents corresponding Palette)
   u16 CurrentAmount;
   s8 AmountPerCall;
   u16 Passes;             // Decrements each call from 0xffff on
}TFadeInfo, *PFadeInfo;


extern TFadeInfo hel_FadeInit(void *pPalette, u8 PalMode, u8 ColMode, u8 PalNo);
extern TFadeInfo hel_FadeInitEx(u16 **pPalette, u8 PaletteMode, u8 ColorMode, u16 Palettes);
extern u8 hel_FadePalette(TFadeInfo *pFadeInfo, s32 AmountPerCall);
extern void hel_FadeReset(TFadeInfo *pFadeInfo);


//
// ---[ SRAM functions ]----------------------------------------------------------------------------
//

extern u8 ATTR_DEPRECATED hel_RAMEntryExists(char *pIdentifier);


//
// ---[ Interrupt functions ]-----------------------------------------------------------------------
//

typedef void (*PInterruptProc)(void);

extern u8 hel_IntrIsEnabled(u32 InterruptType);
extern void hel_IntrStopHandler(u32 InterruptType);
extern void hel_IntrStopAll(void);
extern u32 hel_IntrGetType(void *InterruptProc);
extern void hel_IntrStartHandler(u32 InterruptType, void *pInterruptHandler);

#ifdef HAM_DEBUGGER
    extern void __hel_IntrEmptyVBL(void);
    extern void __hel_IntrEmptyHBL(void);
    extern void __hel_IntrEmptyVCNT(void);
    extern void __hel_IntrEmptyTIM0(void);
    extern void __hel_IntrEmptyTIM1(void);
    extern void __hel_IntrEmptyTIM2(void);
    extern void __hel_IntrEmptyTIM3(void);
    extern void __hel_IntrEmptySIO(void);
    extern void __hel_IntrEmptyDMA0(void);
    extern void __hel_IntrEmptyDMA1(void);
    extern void __hel_IntrEmptyDMA2(void);
    extern void __hel_IntrEmptyDMA3(void);
    extern void __hel_IntrEmptyKEY(void);
    extern void __hel_IntrEmptyCART(void);
#endif // HAM_DEBUGGER


//
// ---[ Bitmap-Mode functions ]---------------------------------------------------------------------
//

extern void hel_BmpClear(u16 Color);
extern u16 *hel_BmpGetBackBuffer(void);
extern void hel_BmpLoad(const void *pSrc);
extern void hel_BmpLoadUnComp(const void *pSrc, u32 CompressionType);

//
// ---[ Timer functions ]---------------------------------------------------------------------------
//

extern void hel_TimerSetFrequency(u8 TimerNo, u8 Value);
extern void hel_TimerStop(u8 TimerNo);
extern void hel_TimerStart(u8 TimerNo);
extern void hel_TimerSetEnable(u8 TimerNo, u8 Value);
extern u16 hel_TimerGetValue(u8 TimerNo);
extern void hel_TimerSetValue(u8 TimerNo, u16 Value);
extern void hel_TimerResetValue(u8 TimerNo);
extern u8 hel_TimerIsEnabled(u8 TimerNo);
extern void hel_TimerSetIRQEnable(u8 TimerNo, u8 Value);
extern u8 hel_TimerIsIRQEnabled(u8 TimerNo);
extern void hel_TimerSetCountUpEnable(u8 TimerNo, u8 Value);
extern u8 hel_TimerIsCountUpEnabled(u8 TimerNo);
extern void hel_TimerStopAll(void);


//
// ---[ HEL Assertion Implementation ]--------------------------------------------------------------
//

#ifdef HAM_DEBUGGER

  typedef struct
  {
      u8 Width;
      u8 PitchY;
      const char CharData[5][8];
  }TAssertCharInfo;

  extern const TAssertCharInfo __hel_AssertCharInfos[];
  extern void hel_AssertImplementation(u32 Assertion, const char *pExpression, const char *pFilename,
                                       const char *pFunction, u32 LineNo, const char *pMessage, ...);

  #define HEL_ASSERT(expression, message) {\
    if(!(expression))\
    {\
			static const char ATTR_SECTION(".rodata") Expression[]=#expression;\
			static const char ATTR_SECTION(".rodata") Message[]=message;\
			static const char ATTR_SECTION(".rodata") File[]=__FILE__;\
			static const char ATTR_SECTION(".rodata") PrettyFunction[]=__PRETTY_FUNCTION__;\
      hel_AssertImplementation((expression), Expression, File, PrettyFunction, __LINE__, Message);\
    }} 
#else

  #define HEL_ASSERT(assertion, message)

#endif // HAM_DEBUGGER


//
// ---[ Debug Message Functions ]-------------------------------------------------------------------
//

#ifdef HAM_DEBUGGER
  extern void hel_DebugNoCashMessage(char *pText, ...);
  extern void hel_DebugNoCashMessageImplementation(char *pText); // helnocashdbgmsgimplementation.s
#endif


//
// ---[ System Functions ]--------------------------------------------------------------------------
//

#define _hel_SysSetPrefetch(Value)    { R_WAITCNT &= NOT_BIT14; R_WAITCNT |= ((Value) << 14); }

#ifdef HAM_DEBUGGER
  extern void hel_SysSetPrefetch(u32 Value);
#else
  #define hel_SysSetPrefetch(Value)   _hel_SysSetPrefetch(Value)
#endif


//
// ---[ Window Functions ]-----------------------------------------------------------------------
//

// R_WIN1Y is wrong in HAM 2.8, so we undef it
// and define a new one, which points to the 
// correct memory location!
#undef  R_WIN1Y
#define R_WIN1Y		*(volatile u16 *)(MEM_IO + 0x46)

#define __hel_WinInit(WinNo, Left, Top, Right, Bottom, InsideFlags, OutsideFlags) hel_WinInitEx(WinNo, Left, Top, Right, Bottom, InsideFlags, OutsideFlags, FALSE)
#define __hel_WinSetRegion(WinNo, Left, Top, Right, Bottom) { M_WINXX_SET((WinNo), (Left), (Right)); M_WINXY_SET((WinNo), (Top), (Bottom)); }
#define __hel_WinDeInit(WinNo)                              __hel_WinHide(WinNo);
#define __hel_WinSetInsideFlags(WinNo, InsideFlags)         { R_WININ &= (0xff00>>((WinNo)<<3)); R_WININ |= ((InsideFlags)<<((WinNo)<<3)); }
#define __hel_WinSetOutsideFlags(WinNo, OutsideFlags)       { R_WINOUT &= (0xff00>>((WinNo)<<3)); R_WINOUT |= (OutsideFlags); }
#define __hel_WinShow(WinNo)                                { R_DISCNT |= BIT(13+(WinNo)); }
#define __hel_WinHide(WinNo)                                { R_DISCNT &= BIT_NOT(13+(WinNo)); }
#define __hel_WinSetVisible(WinNo, Value)                   { R_DISCNT &= BIT_NOT(13+(WinNo)); R_DISCNT |= ((Value) << BIT(13+(WinNo))); }


#ifdef HAM_DEBUGGER
  extern void hel_WinInit(u8 WinNo, u8 Left, u8 Top, u8 Right, u8 Bottom, u8 InsideFlags, u8 OutsideFlags);
  extern void hel_WinDeInit(u8 WinNo);
  extern void hel_WinSetInsideFlags(u8 WinNo, u8 InsideFlags);
  extern void hel_WinSetOutsideFlags(u8 WinNo, u8 OutsideFlags);
  extern void hel_WinSetRegion(u8 WinNo, u8 Left, u8 Top, u8 Right, u8 Bottom);
  extern void hel_WinShow(u8 WinNo);
  extern void hel_WinHide(u8 WinNo);
  extern void hel_WinSetVisible(u8 WinNo, u8 Value);
#else
  #define hel_WinInit(WinNo, Left, Top, Right, Bottom, InsideFlags, OutsideFlags) __hel_WinInit(WinNo, Left, Top, Right, Bottom, InsideFlags, OutsideFlags)
  #define hel_WinDeInit(WinNo)                                __hel_WinDeInit(WinNo)
  #define hel_WinSetInsideFlags(WinNo, InsideFlags)           __hel_WinSetInsideFlags(WinNo, InsideFlags)
  #define hel_WinSetOutsideFlags(WinNo, OutsideFlags)         __hel_WinSetOutsideFlags(WinNo, OutsideFlags)
  #define hel_WinSetRegion(WinNo, Left, Top, Right, Bottom)   __hel_WinSetRegion(WinNo, Left, Top, Right, Bottom)
  #define hel_WinShow(WinNo)                                  __hel_WinShow(WinNo)
  #define hel_WinHide(WinNo)                                  __hel_WinHide(WinNo)
  #define hel_WinSetVisible(WinNo, Value)                     __hel_WinSetVisible(WinNo, Value)
#endif

extern void hel_WinInitEx(u8 WinNo, u8 Left, u8 Top, u8 Right, u8 Bottom, u8 InsideFlags, u8 OutsideFlags, u8 SkipOutsideFlags);


//
// ---[ Special Effect Functions ]-----------------------------------------------------------------------
//
#define hel_FxSetMode(SourceLayer, TargetLayer, FxMode)       R_BLDCNT = (SourceLayer) | ((FxMode)<<6) | ((TargetLayer)<<8)
#define hel_FxSetAlphaLevel(SourceIntensity, TargetIntensity) R_BLDALPHA = (SourceIntensity) | ((TargetIntensity)<<8)
#define hel_FxSetBrightnessLevel(Intensity)                   R_BLDY = (Intensity)


//
// ---[ BIOS Functions ]-------------------------------------------------------------------------------
//
typedef void (*PDecompressFunc)(void *pSrc, void *pDst);

#define COMPRESSION_TYPE_LZ77VRAM         0
#define COMPRESSION_TYPE_LZ77WRAM         1
#define COMPRESSION_TYPE_RLEVRAM          2
#define COMPRESSION_TYPE_RLEWRAM          3

#define __hel_SwiUnComp(pDst, pSrc, CompressionType)  __helSwiDecompressJumpTable[CompressionType]((void*)(pSrc), (void*)(pDst))
#define __hel_SwiLZ77UnCompVram(pDst, pSrc)           __hel_SwiLZ77UnCompVramImplementation((void*)(pSrc), (void*)(pDst))
#define __hel_SwiLZ77UnCompWram(pDst, pSrc)           __hel_SwiLZ77UnCompWramImplementation((void*)(pSrc), (void*)(pDst))
#define __hel_SwiRLUnCompVram(pDst, pSrc)             __hel_RLUnCompVramImplementation((void*)(pSrc), (void*)(pDst))
#define __hel_SwiRLUnCompWram(pDst, pSrc)             __hel_RLUnCompWramImplementation((void*)(pSrc), (void*)(pDst))

// To stay compatible with CLib memcpy as well 
// as DMA functions: First parameter is pDst, pSrc is second!
#ifdef HAM_DEBUGGER
  extern void hel_SwiUnComp(void *pDst, const void *pSrc, u32 CompressionType);
  extern void hel_SwiLZ77UnCompVram(void *pDst, const void *pSrc);
  extern void hel_SwiLZ77UnCompWram(void *pDst, const void *pSrc);
  extern void hel_SwiRLUnCompVram(void *pDst, const void *pSrc);
  extern void hel_SwiRLUnCompWram(void *pDst, const void *pSrc);
#else
  #define hel_SwiUnComp(pDst, pSrc, CompressionType)  __hel_SwiUnComp(pDst, pSrc, CompressionType)
  #define hel_SwiLZ77UnCompVram(pDst, pSrc)           __hel_SwiLZ77UnCompVram(pDst, pSrc)
  #define hel_SwiLZ77UnCompWram(pDst, pSrc)           __hel_SwiLZ77UnCompWram(pDst, pSrc)
  #define hel_SwiRLUnCompVram(pDst, pSrc)             __hel_SwiRLUnCompVram(pDst, pSrc)
  #define hel_SwiRLUnCompWram(pDst, pSrc)             __hel_SwiRLUnCompVram(pDst, pSrc)
#endif

extern const PDecompressFunc __helSwiDecompressJumpTable[];
extern void ATTR_NOINLINE __hel_SwiLZ77UnCompVramImplementation(void *pSrc, void *pDst);
extern void ATTR_NOINLINE __hel_SwiLZ77UnCompWramImplementation(void *pSrc, void *pDst);
extern void ATTR_NOINLINE __hel_RLUnCompVramImplementation(void *pSrc, void *pDst);
extern void ATTR_NOINLINE __hel_RLUnCompWramImplementation(void *pSrc, void *pDst);


//
// ---[ Math Functions ]--------------------------------------------------------------------------------
//
extern void hel_MathExtractDigits(u32 Value, u8 *pOut, u32 nDigits);


#ifdef __cplusplus
   }
#endif

#endif // HEL_H


/* END OF FILE */

